#	jelnet
#	Copyright (C) 2006 Michal Vaner (michal.vaner@kdemail.net)
#
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	he Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
import re
import string
from sys import stderr
from getpass import getpass

socket = None
expectClose = False
ssl = None

def writeToRawSocket( data ):
	global socket
	socket.send( data )

sendDataFunc = writeToRawSocket

def sendData( data ):
	deb( "Sending: " + data )
	sendDataFunc( data )

def writeSsl( data ):
	global ssl
	while len( data ):
		data = data[ ssl.write( data ) : ]

def readRawSocketData():
	global socket
	return socket.recv( 1024 )

def readSsl():
	global ssl
	return ssl.read()

readDataFunc = readRawSocketData

def readMore():
	data = readDataFunc()
	if len( data ) is 0:
		deb( "Connection closed by other side" )
		global expectClose
		if expectClose:
			sys.exit( 0 )
		else:
			sys.stderr.write( "Connection closed by peer\n" )
			sys.exit( 2 )
	deb( "Received: " + data )
	return data

def deb( what ):
	global debug
	if debug:
		stderr.write( "DEB: " + str( what ) + "\n" )

def warn( what ):
	global debug
	if debug:
		stderr.write( "WARN: " + str( what ) + "\n" )

def getArgValue( prefix ):
	deb( "Getting arg value " + prefix );
	reg = re.compile( "^" + prefix + "=.*", re.L | re.S )
	for arg in sys.argv[ 1: ]:
		if reg.match( arg ):
			return arg[ len( prefix ) + 1 : len( arg ) ]
	return None

def statePrint( info ):
	global stateInfo
	if stateInfo:
		sys.stderr.write( "INF: " + info + "\n" )

def parseParams():
	if ( "--help" in sys.argv[ 1: ] ) or ( "-h" in sys.argv[ 1: ] ):
		print "Jelnet (Jabber Telnet)"
		print "jelnet [[--jid=]node@server] [--password=passwd] [--server=server-override]"
		print "[--old-ssl] [--port=portnumber] [--no-bind] [--debug] [--state-info] [--header]"
		print "[--jid=]node@server\t\tSpecify jid to connect to."
		print "\t\t\t\tIf the resource is not specified, server"
		print "\t\t\t\tprovides one. If the jid is not provided,"
		print "\t\t\t\tthe program will ask for it."
		print "--password=passwd\t\tSpecify the password to use."
		print "--server=server-override\tSpecify different server to use than the one"
		print "\t\t\t\tin jid."
		print "--old-ssl\t\t\tUse old-fashioned ssl encryption."
		print "--port=portnumber\t\tOverride default port number."
		print "--no-bind\t\t\tDo not bind resource and init session."
		print "-d,--debug\t\t\tDebug info"
		print "-i,--state-info\t\t\tPrint connection state, non-human readable info."
		print "--header\t\t\tPrint the stream header for xml parser."
		print "-l,--lines\t\t\tAdd new lines between incoming tags, for readability."
		print "--silent-passwd\t\tDo not prompt for password (but expect it)."
		sys.exit( 0 )
	#Do we debug?
	global debug
	if ( "-d" in sys.argv[ 1: ] ) or ( "--debug" in sys.argv[ 1: ] ):
		debug = True
	else:
		debug = False
	global stateInfo#is state info requested?
	if ( "-i" in sys.argv[ 1: ] ) or ( "--state-info" in sys.argv[ 1: ] ):
		stateInfo = True
	else:
		stateInfo = False
	global doBind
	if ( "--no-bind" in sys.argv[ 1: ] ):
		doBind = False
	else:
		doBind = True
	global lineSplits
	if ( "-l" in sys.argv[ 1: ] ) or ( "--lines" in sys.argv[ 1: ] ):
		lineSplits = True
	else:
		lineSplits = False
	global jid#check if JID is provided
	jid = getArgValue( "--jid" )
	if jid is None:
		for arg in sys.argv[ 1: ]:
			if arg[ 0 ] != '-':
				jid = arg
				break
	global passwd#check if there is a password
	passwd = getArgValue( "-p" )
	if passwd is None:
		passwd = getArgValue( "--password" )
	global server
	server = getArgValue( "--server" )#server override?
	global oldSSL#use old ssl?
	oldSSL = ( "--old-ssl" in sys.argv[ 1: ] )
	sport = getArgValue( "--port" )#port override?
	global port
	if sport != None:
		port = string.atoi( sport )
	else:
		if oldSSL:#default by what the thing is
			port = 5223
		else:
			port = 5222
	global printHeader
	if ( "--header" in sys.argv[ 1: ] ):
		printHeader = True
	else:
		printHeader = False
	global silentPasswd
	if( "--silent-passwd" in sys.argv[ 1: ] ):
		silentPasswd = True
	else:
		silentPasswd = False

def extractServer( jid ):
	mo = re.search( "^.*@(.[^/]*)(/.*)?$", jid )
	if mo:
		return mo.group( 1 )
	else:
		return jid

def checkAllNeeded():
	global jid
	if jid is None:
		jid = raw_input( "JID: " )
	global passwd, silentPasswd
	if passwd is None:
		if silentPasswd:
			passwd = raw_input( "" )
		else:
			passwd = getpass()
	global server
	if server is None:
		server = extractServer( jid )

